'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createContainer;

var _utils = require('./utils');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

/**
 * @typedef   {Object}    Container
 * @property  {Function}  add
 * @property  {Function}  get
 */

/**
 * @return {Container}
 */
function createContainer() {
  var dependencies = [];

  function add(name, func) {
    var args = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

    (0, _utils.validateArguments)([name, func, args], 'add', {
      name: ['string', true],
      constructor: ['function', true],
      dependencies: ['array']
    });
    (0, _utils.invariant)(!findDependency(name), 'Dependency \'' + name + '\' already exists.');

    var dependency = { name: name, func: func, args: args };

    dependencies = [].concat(_toConsumableArray(dependencies), [dependency]);
  }

  function get(name) {
    (0, _utils.validateArguments)([name], 'get', { name: ['string', true] });
    (0, _utils.invariant)(findDependency(name), 'Dependency \'' + name + '\' does not exist.');

    var dependency = findDependency(name);
    var args = dependency.args.map(get);

    return invoke(dependency, args);
  }

  function findDependency(name) {
    return dependencies.find(function (dep) {
      return dep.name === name;
    });
  }

  return { add: add, get: get };
}

function invoke(_ref, args) {
  var func = _ref.func;

  return func.hasOwnProperty('prototype') ? new (Function.prototype.bind.apply(func, [null].concat(_toConsumableArray(args))))() : // eslint-disable-line new-cap
  func.apply(undefined, _toConsumableArray(args));
}